package no.ntnu.jervidal.master;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import no.ntnu.jervidal.master.medline.MyMedlineIndexer;
import no.ntnu.jervidal.master.medline.MyMedlineSearcher;
import no.ntnu.jervidal.master.medline.evaluation.TrecEval;
import no.ntnu.jervidal.master.medline.results.SearchResults;
import no.ntnu.jervidal.master.medline.searchers.LuceneDefault;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.FileAppender;
import org.apache.log4j.Logger;
import org.apache.log4j.PatternLayout;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;

public class Main {

	private static String CMD_CREATE_INDEX = "create_index";
	private static String CMD_SEARCH = "search";
	private static String CMD_EVAL = "trec_eval";
	private static Logger logger;
	public static final Properties properties = new Properties();
	public static final String PROPERTIES_FILE_PATH = "settings.properties";

	// private static MedlineSearcher =

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 0) {
			System.err.println("No arguments");
			System.exit(0);
		}

		try {
			Logger.getRootLogger().addAppender(
					(new FileAppender(new PatternLayout("%d{ISO8601}: %-5p [%t]: %m%n"), "/tmp/masterlogg", false)));
		} catch (IOException e) {
			e.printStackTrace();
		}
		BasicConfigurator.configure();

		logger = Logger.getRootLogger();

		try {
			properties.load(new FileInputStream(PROPERTIES_FILE_PATH));
		} catch (IOException e) {
			logger.error("Could not load properties file.", e);
		}

		if (args[0].equals(CMD_CREATE_INDEX)) {
			logger.info("Creating new index(es).");
			MyMedlineIndexer indexer = new MyMedlineIndexer();
			indexer.index();
		} else if (args[0].equals(CMD_SEARCH)) {
			if (args.length != 2) {
				logger.error("No search query was entered.");
				System.exit(1);
			}
			try {
				MyMedlineSearcher searcher = new MyMedlineSearcher(new LuceneDefault());
				SearchResults results = searcher.search(args[1]);
				results.print();
				// MedlineSearcher searcher2 = new
				// LuceneDefaultMedlineSearcher();
				// TopDocs docs = searcher2.search(args[1]);
				// logger.debug("Found " + docs.totalHits +
				// " documents. The best score was " + docs.getMaxScore());
			} catch (CorruptIndexException e) {
				logger.error("Could not read the index", e);
				System.exit(1);
			} catch (IOException e) {
				e.printStackTrace();
				System.exit(1);
			} catch (ParseException e) {
				logger.error("Invalid search query.", e);
			}
		} else if (args[0].equals(CMD_EVAL)) {
			TrecEval eval = new TrecEval();
			eval.runEvaluation();
		}

	}
}
