package no.ntnu.jervidal.master.medline;

import java.io.IOException;

import no.ntnu.jervidal.master.medline.indexers.MedlineIndexWriter;

import org.apache.log4j.Logger;
import org.apache.lucene.index.CorruptIndexException;

import com.aliasi.lingmed.parser.Abstract;
import com.aliasi.lingmed.parser.MedlineCitation;
import com.aliasi.lingmed.parser.MedlineHandler;

public class MyMedlineHandler implements MedlineHandler {

	private MedlineIndexWriter indexWriter;
	private Logger logger;

	public MyMedlineHandler(MedlineIndexWriter indexWriter) {
		this.indexWriter = indexWriter;
		logger = Logger.getLogger(this.getClass());
	}

	@Override
	public void delete(String pmid) {
		logger.error("Unhandled delete with pmid " + pmid + ".");
	}

	@Override
	public void handle(MedlineCitation citation) {
		Abstract abstrct = citation.article().abstrct();
		if (abstrct == null) {
			logger.warn("Document with pmid " + citation.pmid() + " had no abstract. Skipping.");
			return;
		}
		try {
			indexWriter.addDocument(citation.article().articleTitleText(), abstrct.textWithoutTruncationMarker(),
					citation.pmid(), citation.meshHeadings());
		} catch (CorruptIndexException e) {
			logger.error("Could not access the index.", e);
		} catch (IOException e) {
			logger.error("An error occured when adding the artice with pmid " + citation.pmid() + " to the index.", e);
		}
	}

}
