package no.ntnu.jervidal.master.medline;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.indexers.BM25FIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25GeneTagAndMeshIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25GeneTagGeniaAndMeshIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25GeneTagIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25GeniaAndMeshIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25GeniaIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.BM25MeshIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.LuceneDefaultIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.MedlineIndexWriter;
import no.ntnu.jervidal.master.medline.indexers.MeshIndexWriter;

import org.apache.log4j.Logger;
import org.apache.lucene.index.CorruptIndexException;

import com.aliasi.lingmed.parser.MedlineParser;

public class MyMedlineIndexer {

	private Logger logger = Logger.getLogger(this.getClass());
	private MedlineIndexWriter[] indexAlgorithms = {
			new LuceneDefaultIndexWriter(), new BM25FIndexWriter(), new BM25GeneTagIndexWriter(),
			new BM25GeneTagAndMeshIndexWriter(), new BM25GeniaIndexWriter(), new MeshIndexWriter(),
			new BM25GeniaAndMeshIndexWriter(), new BM25GeneTagGeniaAndMeshIndexWriter(), new BM25MeshIndexWriter(),
	};

	public void index() {
		for (MedlineIndexWriter indexer : indexAlgorithms) {
			try {
				logger.info("Start indexing using " + indexer.getClass().getSimpleName() + ".");
				Date startTime = new Date();
				MedlineParser parser = new MedlineParser(false);
				parser.setHandler(new MyMedlineHandler(indexer));
				File medlineDir = new File(Main.properties.getProperty("medlineDirectory"));
				if (!medlineDir.isDirectory()) {
					throw new FileNotFoundException("\"" + medlineDir + "\" is not a directory.");
				}
				for (File xmlFile : medlineDir.listFiles()) {
					if (xmlFile.getPath().toLowerCase().endsWith(".xml")) {
						parser.parse(xmlFile);
					}
				}
				Calendar runningTime = Calendar.getInstance();
				runningTime.setTimeInMillis(System.currentTimeMillis() - startTime.getTime());
				logger.info("Indexed " + indexer.getNumberOfDocsInIndex() + " documents in "
						+ runningTime.get(Calendar.HOUR) + " hours and " + runningTime.get(Calendar.MINUTE) + ":"
						+ runningTime.get(Calendar.SECOND) + "," + runningTime.get(Calendar.MILLISECOND) + " minutes.");
				indexer.optimize();
				indexer.close();
			} catch (CorruptIndexException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}
}
