package no.ntnu.jervidal.master.medline;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.results.SearchResults;
import no.ntnu.jervidal.master.medline.searchers.MedlineSearcher;
import no.ntnu.jervidal.master.medline.searchers.MeshPhrase;

import org.apache.log4j.Logger;
import org.apache.lucene.queryParser.ParseException;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.aliasi.chunk.Chunk;
import com.aliasi.chunk.Chunker;
import com.aliasi.util.AbstractExternalizable;

public class MyMedlineSearcher {

	private Logger logger = Logger.getLogger(this.getClass());
	private MedlineSearcher searchAlgorithm;

	public MyMedlineSearcher(MedlineSearcher algorithm) {
		this.searchAlgorithm = algorithm;
		this.searchAlgorithm.initialize();
	}

	/**
	 * Performs several searches using queries found in a file defined by
	 * <code>queriesPath</code> in the settings file. Each line is read as a
	 * single query.
	 * 
	 * @return A {@link List} of {@link SearchResults}, one for each query.
	 * @throws ParseException
	 *             if the query file contains invalid queries.
	 * @throws IOException
	 *             if a low-level IO error occured.
	 */
	public List<SearchResults> search() throws ParseException, IOException {
		logger.info("Starting search using " + searchAlgorithm.getClass().getSimpleName() + ".");
		List<SearchResults> results = new LinkedList<SearchResults>();
		if (!(searchAlgorithm instanceof MeshPhrase)) {
			File queries = new File(Main.properties.getProperty("queriesPath"));
			try {
				Scanner scanner = new Scanner(queries);
				while (scanner.hasNextLine()) {
					String next = scanner.nextLine();
					if (!next.startsWith("#"))
						results.add(search(next));
				}
				return results;
			} catch (FileNotFoundException e) {
				logger.error("Could not read the query file \"" + queries + "\".", e);
				return null;
			}
		} else {
			File queries = new File(Main.properties.getProperty("fulltextQueriesPath"));
			DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
			dbf.setNamespaceAware(true);
			try {
				Chunker chunker = (Chunker) AbstractExternalizable.readObject(new File(Main.properties
						.getProperty("GeneTagPath")));
				Chunker chunkerAlternative = (Chunker) AbstractExternalizable.readObject(new File(Main.properties
						.getProperty("GeniaPath")));
				DocumentBuilder db = dbf.newDocumentBuilder();
				Document doc = db.parse(queries);
				XPathFactory xpFactory = XPathFactory.newInstance();
				XPath xPath = xpFactory.newXPath();
				XPathExpression expr = xPath.compile("TOPICS/TOPIC/NEED/text()");
				NodeList nodesNeed = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
				expr = xPath.compile("TOPICS/TOPIC/CONTEXT/text()");
				NodeList nodesContext = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
				expr = xPath.compile("TOPICS/TOPIC/TITLE/text()");
				NodeList nodesTitle = (NodeList) expr.evaluate(doc, XPathConstants.NODESET);
				if ((nodesNeed.getLength() != nodesContext.getLength()) && nodesNeed.getLength() != 50)
					logger.warn("There were " + nodesNeed.getLength()
							+ " \"NEED\"-nodes in the XML file (should be 50)!");
				for (int i = 0; i < nodesNeed.getLength(); i++) {
					String query = nodesTitle.item(i).getNodeValue() + ". " + nodesNeed.item(i).getNodeValue() + " "
							+ nodesContext.item(i).getNodeValue();
					char[] queryArray = query.toCharArray();
					HashSet<String> querySet = new HashSet<String>();
					for (Chunk chunk : (chunker.chunk(queryArray, 0, queryArray.length).chunkSet())) {
						querySet.add(query.substring(chunk.start(), chunk.end()).replaceAll("[;()]", ""));
					}
					if (querySet.size() == 0) {
						logger.warn("The query '" + query + "' did not chunk.");
						for (Chunk chunk : (chunkerAlternative.chunk(queryArray, 0, queryArray.length).chunkSet())) {
							querySet.add(query.substring(chunk.start(), chunk.end()).replaceAll("[;()]", ""));
						}
					}
					StringBuilder sb = new StringBuilder();
					for (String s : querySet)
						sb.append("(" + s.replaceAll(" ", " AND ") + ") ");
					System.out.println(sb.toString());
					results.add(search(sb.toString().trim()));
				}
				return results;
			} catch (ParserConfigurationException e) {
				logger.error("Could not get the DocumentBuilder...", e);
			} catch (SAXException e) {
				logger.error("Could not parse the document.", e);
			} catch (ClassNotFoundException e) {
				logger.error("Could not load the chunker.", e);
			} catch (XPathExpressionException e) {
				logger.error("Could not parse the XPath expression.", e);
			}
			return results;
		}
	}

	public SearchResults search(String query) throws ParseException, IOException {
		return new SearchResults(searchAlgorithm, searchAlgorithm.search(query));
	}
}
