package no.ntnu.jervidal.master.medline.analyzers;

import java.io.File;
import java.io.IOException;

import no.ntnu.jervidal.master.Main;

import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.standard.StandardAnalyzer;
import org.apache.lucene.util.Version;

public class AnalyzerFactory {

	private static Logger logger = Logger.getLogger(StandardAnalyzer.class);
	private static StandardAnalyzer standarMedlineAnalyzer;
	private static Analyzer bm25MedlineAnalyzer;

	/**
	 * Creates and returns a new {@link StandardAnalyzer}, but using the
	 * recommended stopword list for Medline if found. If an analyzer already
	 * exists, it will be reused.
	 * 
	 * @return a {@link StandardAnalyzer} with a stopword list better suited for
	 *         Medline searches. If no stopword list is found, the default list
	 *         is used.
	 */
	public static StandardAnalyzer getStandardMedlineAnalyzer() {
		if (standarMedlineAnalyzer != null)
			return standarMedlineAnalyzer;
		File stopwords = new File(Main.properties.getProperty("stopwordsPath"));

		if (stopwords.canRead()) {
			logger.debug("Loading stopwords from file " + stopwords);
			try {
				standarMedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30, stopwords);
			} catch (IOException e) {
				logger.debug("The file \"" + stopwords + "\" was not valid. Using default stopword list instead.", e);
				standarMedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30);
			}
		} else {
			logger.debug("Using default stopwords");
			standarMedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30);
		}
		return standarMedlineAnalyzer;
	}

	public static Analyzer getBM25MedlineAnalyzer() {
		// if (bm25MedlineAnalyzer != null)
		// return bm25MedlineAnalyzer;
		// File stopwords = new
		// File(Main.properties.getProperty("stopwordsPath"));
		//
		// if (stopwords.canRead()) {
		// logger.debug("Loading stopwords from file " + stopwords);
		// try {
		// bm25MedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30,
		// stopwords);
		// } catch (IOException e) {
		// logger.debug("The file \"" + stopwords +
		// "\" was not valid. Using default stopword list instead.", e);
		// bm25MedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30);
		// }
		// } else {
		// logger.debug("Using default stopwords");
		// bm25MedlineAnalyzer = new StandardAnalyzer(Version.LUCENE_30);
		// }
		// return bm25MedlineAnalyzer;
		return getStandardMedlineAnalyzer();
	}
}
