package no.ntnu.jervidal.master.medline.analyzers.bm25;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.TermDocs;
import org.apache.lucene.search.Scorer;
import org.apache.lucene.search.Similarity;

public class BM25Scorer extends Scorer {

	private Logger logger = Logger.getLogger(this.getClass());
	private BM25TermQuery term;
	private TermDocs termDocs;
	private IndexReader indexReader;
	private float b;
	private float k1;
	private float idf;
	private int doc = -1;
	private float avgLength;
	private byte[] norm;

	protected BM25Scorer(Similarity similarity, BM25TermQuery term, IndexReader indexReader) throws IOException {
		super(similarity);
		this.term = term;
		this.indexReader = indexReader;
		this.b = BM25Parameters.getB();
		this.k1 = BM25Parameters.getK1();
		this.norm = indexReader.norms(term.getTerm().field());
		this.idf = similarity.idf(indexReader.docFreq(term.getTerm()), indexReader.numDocs());
		this.termDocs = indexReader.termDocs(term.getTerm());
		this.avgLength = BM25Parameters.getAverageLength(term.getTerm().field());
	}

	@Override
	public float score() throws IOException {
		float fieldNorm = Similarity.decodeNorm(norm[docID()]);
		float length = 1 / (fieldNorm * fieldNorm);

		// LENGTH NORMALIZATION
		float result = b * (length / avgLength);
		result += 1 - b;

		result = (term.getBoost() * termDocs.freq()) / result;
		// FREQ SATURATION
		result /= (result + k1);

		return result * idf;
		// Document document = indexReader.document(doc);
		// int docLen = 0;
		// if
		// (term.getTerm().field().equals(Main.properties.getProperty("docFieldTitle")))
		// {
		// docLen =
		// Integer.parseInt(document.get(Main.properties.getProperty("docTitleLength")));
		// } else if
		// (term.getTerm().field().equals(Main.properties.getProperty("docFieldAbstract")))
		// {
		// docLen =
		// Integer.parseInt(document.get(Main.properties.getProperty("docAbstractLength")));
		// }
		// // logger.debug(idf + " * (" + k1 + " + " + 1f + ") * " +
		// // termDocs.freq() + " / (" + k1 + " * ((1 - " + b + ") + "
		// // + b + " * " + docLen + " / " + avgLength + ") + " +
		// termDocs.freq());
		// return idf * (k1 + 1f) * termDocs.freq() / (k1 * ((1 - b) + b *
		// docLen / avgLength) + termDocs.freq());
	}

	@Override
	public int advance(int target) throws IOException {
		if (target == NO_MORE_DOCS)
			return NO_MORE_DOCS;
		while (nextDoc() != NO_MORE_DOCS && docID() < target) {
		}
		return docID();
	}

	@Override
	public int docID() {
		return doc;
	}

	@Override
	public int nextDoc() throws IOException {
		boolean result = termDocs.next();
		if (result) {
			doc = termDocs.doc();
		} else {
			termDocs.close();
			doc = NO_MORE_DOCS;
		}
		return doc;
	}

}