package no.ntnu.jervidal.master.medline.analyzers.bm25;

/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.util.HashMap;
import java.util.Map;

import org.apache.lucene.search.DefaultSimilarity;

/**
 * Similarity for BM25 ranking functions family.<BR>
 * IDF is implemented as:<BR>
 * log ((N-n+0.5)/(n+0.5))<BR>
 * where n = docFreq(term) and N = numDocs().
 * 
 * 
 * 
 */
public class BM25Similarity extends DefaultSimilarity {

	private static Map<String, Long> length = new HashMap<String, Long>();

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.lucene.search.Similarity#coord(int, int)
	 */
	@Override
	public float coord(int overlap, int maxOverlap) {
		return (float) overlap / (float) maxOverlap;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.lucene.search.Similarity#idf(int, int)
	 */
	@Override
	public float idf(int docFreq, int numDocs) {
		float result = (numDocs - docFreq + 0.5f);
		result = result / (docFreq + 0.5f);
		return (float) Math.log(result);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.lucene.search.Similarity#lengthNorm(java.lang.String,
	 * int)
	 */
	@Override
	public float lengthNorm(String fieldName, int numTokens) {
		Long aux = length.get(fieldName);
		if (aux == null)
			aux = new Long(0);
		aux += numTokens;
		length.put(fieldName, aux);
		return super.lengthNorm(fieldName, numTokens);
	}

	public static long getLength(String field) {
		return length.get(field);
	}

	public static void resetLengthNorm() {
		length.clear();
	}

	// /*
	// * (non-Javadoc)
	// *
	// * @see org.apache.lucene.search.Similarity#queryNorm(float)
	// */
	// @Override
	// public float queryNorm(float sumOfSquaredWeights) {
	//
	// return 1;
	// }

	// /*
	// * (non-Javadoc)
	// *
	// * @see org.apache.lucene.search.Similarity#sloppyFreq(int)
	// */
	// @Override
	// public float sloppyFreq(int distance) {
	//
	// return 1;
	// }

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.apache.lucene.search.Similarity#tf(float)
	 */
	@Override
	public float tf(float freq) {

		return freq;
	}

}
