package no.ntnu.jervidal.master.medline.analyzers.bm25;

import java.io.IOException;

import org.apache.log4j.Logger;
import org.apache.lucene.index.IndexReader;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.Explanation;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Scorer;
import org.apache.lucene.search.Searcher;
import org.apache.lucene.search.Similarity;
import org.apache.lucene.search.Weight;
import org.apache.lucene.util.ToStringUtils;

@SuppressWarnings("serial")
public class BM25TermQuery extends Query {

	private Logger logger = Logger.getLogger(this.getClass());
	private Term term;
	private String[] fields = null;
	private float[] boosts;
	private float[] bParams;

	public BM25TermQuery(Term term) {
		this.term = term;
	}

	public BM25TermQuery(Term term, String[] fields, float[] boosts, float[] bParams) {
		this.term = term;
		this.fields = fields;
		this.boosts = boosts;
		this.bParams = bParams;
	}

	public Term getTerm() {
		return term;
	}

	protected class BM25Weight extends Weight {

		private Similarity similarity;
		private float idf;

		// private String[] fields = null;
		// private float[] boosts;
		// private float[] bParams;

		public BM25Weight(Searcher searcher) throws IOException {
			similarity = getSimilarity(searcher);
			idf = similarity.idf(searcher.docFreq(term), searcher.maxDoc());
		}

		@Override
		public Explanation explain(IndexReader reader, int doc) throws IOException {
			// logger.debug("Called BM25Query.BM25Weight.explain(). Doing nothing.");
			return null;
		}

		@Override
		public Query getQuery() {
			return BM25TermQuery.this;
		}

		@Override
		public float getValue() {
			// logger.debug("Called BM25Query.BM25Weight.getValue(). Returning 0.");
			return 0;
		}

		@Override
		public void normalize(float norm) {
			// logger.debug("Called BM25Query.BM25Weight.normalize(). Doing nothing.");
		}

		@Override
		public Scorer scorer(IndexReader reader, boolean scoreDocsInOrder, boolean topScorer) throws IOException {
			if (fields == null)
				return new BM25Scorer(similarity, BM25TermQuery.this, reader);
			else
				// return new BM25FScorer(similarity, BM25TermQuery.this,
				// reader, fields, boosts, bParams);
				return new BM25FScorerWithMesh(similarity, BM25TermQuery.this, reader, fields, boosts, bParams);
		}

		@Override
		public float sumOfSquaredWeights() throws IOException {
			float queryWeight = idf * getBoost(); // compute query weight
			return queryWeight * queryWeight; // square it
		}

	}

	@Override
	public Weight createWeight(Searcher searcher) throws IOException {
		return new BM25Weight(searcher);
	}

	@Override
	public String toString(String field) {
		StringBuffer buffer = new StringBuffer();
		if (!term.field().equals(field)) {
			buffer.append(term.field());
			buffer.append(":");
		}
		buffer.append(term.text());
		buffer.append(ToStringUtils.boost(getBoost()));
		return buffer.toString();
	}

}
