package no.ntnu.jervidal.master.medline.indexers;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.AnalyzerFactory;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25Similarity;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.aliasi.lingmed.parser.MeshHeading;

public class BM25FIndexWriter extends MedlineIndexWriter {

	@Override
	protected IndexWriter initializeIndexWriter() throws IOException {
		String dirString = Main.properties.getProperty("indexDirectory");
		Directory dir = FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryBM25")));
		IndexWriter writer = new IndexWriter(dir, AnalyzerFactory.getBM25MedlineAnalyzer(), true,
				IndexWriter.MaxFieldLength.LIMITED);
		writer.setSimilarity(new BM25Similarity());
		return writer;
	}

	@Override
	public void close() throws CorruptIndexException, IOException {
		super.close();
		float titleLength = BM25Similarity.getLength(Main.properties.getProperty("docFieldTitle", "title"))
				/ getNumberOfDocsInIndex();
		float abstractLength = BM25Similarity.getLength(Main.properties.getProperty("docFieldAbstract", "abstract"))
				/ getNumberOfDocsInIndex();
		logger.info("Average title length: " + titleLength + " - Average abstract length: " + abstractLength);
		Main.properties.setProperty("BM25AverageTitleLength", titleLength + "");
		Main.properties.setProperty("BM25AverageAbstractLength", abstractLength + "");
		Main.properties.store(new FileWriter(Main.PROPERTIES_FILE_PATH), null);
		BM25Similarity.resetLengthNorm();
	}

	@Override
	public void addDocument(String titleText, String abstrText, String pmid, MeshHeading[] meshHeadings)
			throws CorruptIndexException, IOException {
		Document doc = new Document();

		Field titleField = new Field(Main.properties.getProperty("docFieldTitle"), titleText, Field.Store.YES,
				Field.Index.ANALYZED);
		Field pmidField = new Field(Main.properties.getProperty("docFieldPMID"), pmid, Field.Store.YES, Field.Index.NO);
		Field abstractField = new Field(Main.properties.getProperty("docFieldAbstract"), abstrText, Field.Store.NO,
				Field.Index.ANALYZED);
		Field titleLengthField = new Field(Main.properties.getProperty("docTitleLength"), countWords(titleText) + "",
				Field.Store.YES, Field.Index.NO);
		Field abstractLengthField = new Field(Main.properties.getProperty("docAbstractLength"), countWords(abstrText)
				+ "", Field.Store.YES, Field.Index.NO);

		doc.add(titleField);
		doc.add(pmidField);
		doc.add(abstractField);
		doc.add(titleLengthField);
		doc.add(abstractLengthField);

		indexWriter.addDocument(doc);
	}

	protected int countWords(String string) {
		String[] divided = string.split("\\W");
		return divided.length;
	}

	public static float calculateFieldBoost(float k1, float b, int length, float averageLength, float chunkSize) {
		return ((k1 + 1) * (chunkSize + 1)) / (k1 * (1 - b + (b * (length / averageLength))) + (chunkSize + 1));
	}
}
