package no.ntnu.jervidal.master.medline.indexers;

import java.io.File;
import java.io.IOException;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.AnalyzerFactory;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25FParameters;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25Similarity;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.lingmed.parser.MeshHeading;
import com.aliasi.util.AbstractExternalizable;

public class BM25GeneTagIndexWriter extends BM25FIndexWriter {

	private Chunker chunker;
	private float averageTitleLength;
	private float averageAbstractLength;

	@Override
	protected IndexWriter initializeIndexWriter() throws IOException {
		File chunkFile = new File(Main.properties.getProperty("GeneTagPath"));
		try {
			chunker = (Chunker) AbstractExternalizable.readObject(chunkFile);
		} catch (ClassNotFoundException e) {
			logger.error("Could not load the chunker.", e);
		}
		String dirString = Main.properties.getProperty("indexDirectory");
		Directory dir = FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryBM25GeneTag")));
		IndexWriter writer = new IndexWriter(dir, AnalyzerFactory.getBM25MedlineAnalyzer(), true,
				IndexWriter.MaxFieldLength.LIMITED);
		writer.setSimilarity(new BM25Similarity());
		averageTitleLength = Float.parseFloat(Main.properties.getProperty("BM25AverageTitleLength", "10.0"));
		averageAbstractLength = Float.parseFloat(Main.properties.getProperty("BM25AverageAbstractLength", "119.0"));
		return writer;
	}

	@Override
	public void addDocument(String titleText, String abstrText, String pmid, MeshHeading[] meshHeadings)
			throws CorruptIndexException, IOException {
		Document doc = new Document();

		int titleLength = countWords(titleText);
		int abstractLength = countWords(abstrText);

		Field titleField = new Field(Main.properties.getProperty("docFieldTitle"), titleText, Field.Store.YES,
				Field.Index.ANALYZED);
		Field pmidField = new Field(Main.properties.getProperty("docFieldPMID"), pmid, Field.Store.YES, Field.Index.NO);
		Field abstractField = new Field(Main.properties.getProperty("docFieldAbstract"), abstrText, Field.Store.NO,
				Field.Index.ANALYZED);
		Field titleLengthField = new Field(Main.properties.getProperty("docTitleLength"), titleLength + "",
				Field.Store.YES, Field.Index.NO);
		Field abstractLengthField = new Field(Main.properties.getProperty("docAbstractLength"), abstractLength + "",
				Field.Store.YES, Field.Index.NO);

		Chunking titleChunk = chunker.chunk(titleText);
		Chunking abstractChunk = chunker.chunk(abstrText);

		float k1 = BM25FParameters.getK1();
		float b = BM25FParameters.getB();
		titleField.setBoost(calculateFieldBoost(k1, b, titleLength, averageTitleLength, titleChunk.chunkSet().size()));
		abstractField.setBoost(calculateFieldBoost(k1, b, abstractLength, averageAbstractLength, abstractChunk
				.chunkSet().size()));

		doc.add(titleField);
		doc.add(pmidField);
		doc.add(abstractField);
		doc.add(titleLengthField);
		doc.add(abstractLengthField);

		indexWriter.addDocument(doc);
	}

}
