package no.ntnu.jervidal.master.medline.indexers;

import java.io.File;
import java.io.IOException;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.AnalyzerFactory;

import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import com.aliasi.lingmed.parser.MeshHeading;
import com.aliasi.lingmed.parser.Topic;

public class MeshIndexWriter extends MedlineIndexWriter {

	@Override
	protected IndexWriter initializeIndexWriter() throws IOException {
		String dirString = Main.properties.getProperty("indexDirectory");
		Directory dir = FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryMesh")));
		IndexWriter writer = new IndexWriter(dir, AnalyzerFactory.getStandardMedlineAnalyzer(), true,
				IndexWriter.MaxFieldLength.LIMITED);
		return writer;
	}

	@Override
	public void addDocument(String titleText, String abstrText, String pmid, MeshHeading[] meshHeadings)
			throws CorruptIndexException, IOException {
		Document doc = new Document();

		Field titleField = new Field(Main.properties.getProperty("docFieldTitle"), titleText, Field.Store.YES,
				Field.Index.ANALYZED);
		Field pmidField = new Field(Main.properties.getProperty("docFieldPMID"), pmid, Field.Store.YES, Field.Index.NO);
		Field abstractField = new Field(Main.properties.getProperty("docFieldAbstract"), abstrText, Field.Store.NO,
				Field.Index.ANALYZED);
		StringBuilder sb = new StringBuilder();
		for (MeshHeading mesh : meshHeadings) {
			for (Topic topic : mesh.topics()) {
				sb.append(topic.topic() + " ");
			}
		}
		Field meshField = new Field(Main.properties.getProperty("docFieldMesh"), sb.toString(), Field.Store.NO,
				Field.Index.ANALYZED);

		doc.add(titleField);
		doc.add(pmidField);
		doc.add(abstractField);
		doc.add(meshField);

		indexWriter.addDocument(doc);
	}

}
