package no.ntnu.jervidal.master.medline.results;

import java.io.IOException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.searchers.MedlineSearcher;

import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TopDocs;

public class SearchResults implements Iterable<SearchResult> {
	private List<SearchResult> searchResults;
	private int totalHits;

	public SearchResults(MedlineSearcher searcher, TopDocs topDocs) throws CorruptIndexException, IOException {
		totalHits = topDocs.totalHits;
		searchResults = new LinkedList<SearchResult>();
		int rank = 1;
		for (ScoreDoc scoreDoc : topDocs.scoreDocs) {
			searchResults.add(new SearchResult(rank, searcher.getDocument(scoreDoc.doc), scoreDoc.score));
			rank++;
		}
	}

	public void print() {
		for (SearchResult result : searchResults) {
			Document doc = result.getDocument();
			String title = doc.get(Main.properties.getProperty("docFieldTitle", "title"));
			String pmid = doc.get(Main.properties.getProperty("docFieldPMID", "pmid"));
			System.out.println(result.getRank() + " - " + pmid + " - " + result.getScore() + " - " + title);
		}
		System.out.println("\nTotal hits: " + totalHits);
	}

	@Override
	public Iterator<SearchResult> iterator() {
		return searchResults.iterator();
	}
}
