package no.ntnu.jervidal.master.medline.searchers;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25FParameters;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25Similarity;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.store.FSDirectory;

public class BM25GeneTagGeniaAndMesh extends BM25 {

	@Override
	public IndexSearcher initializeIndexSearcher() throws CorruptIndexException, IOException {
		String dirString = Main.properties.getProperty("indexDirectory");
		IndexSearcher searcher = new IndexSearcher(FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryBM25GeniaGeneTagAndMesh"))));
		BM25FParameters.setAverageLength(Main.properties.getProperty("docFieldTitle"), Float.parseFloat(Main.properties
				.getProperty("BM25AverageTitleLength", "10.0")));
		BM25FParameters.setAverageLength(Main.properties.getProperty("docFieldAbstract"), Float
				.parseFloat(Main.properties.getProperty("BM25AverageAbstractLength", "119.0")));
		BM25FParameters.setAverageLength(Main.properties.getProperty("docFieldMesh"), Float.parseFloat(Main.properties
				.getProperty("BM25AverageMeshLength")));
		BM25FParameters.setK1(2.0f);
		searcher.setSimilarity(new BM25Similarity());
		fields = new String[] {
				Main.properties.getProperty("docFieldTitle", "title"),
				Main.properties.getProperty("docFieldAbstract", "abstract"),
				Main.properties.getProperty("docFieldMesh", "mesh")
		};
		float[] bParams = new float[fields.length];
		for (int i = 0; i < fields.length; i++) {
			bParams[i] = 0.75f;
		}
		BM25FParameters.setBParam(bParams);
		boosts = new HashMap<String, Float>();
		boosts.put(fields[0], Float.parseFloat(Main.properties.getProperty("titleBoost")));
		boosts.put(fields[1], Float.parseFloat(Main.properties.getProperty("abstractBoost")));
		boosts.put(fields[2], Float.parseFloat(Main.properties.getProperty("meshBoost")));
		return searcher;
	}

}
