package no.ntnu.jervidal.master.medline.searchers;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25FParameters;
import no.ntnu.jervidal.master.medline.analyzers.bm25.BM25Similarity;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.store.FSDirectory;

import com.aliasi.chunk.Chunker;
import com.aliasi.chunk.Chunking;
import com.aliasi.util.AbstractExternalizable;

public class BM25Mesh extends BM25 {

	private Chunker chunker;
	private Chunking chunking;

	@Override
	public IndexSearcher initializeIndexSearcher() throws CorruptIndexException, IOException {
		File chunkFile = new File(Main.properties.getProperty("MeshPath"));
		try {
			chunker = (Chunker) AbstractExternalizable.readObject(chunkFile);
		} catch (ClassNotFoundException e) {
			logger.error("Could not load the chunker.", e);
		}
		String dirString = Main.properties.getProperty("indexDirectory");
		IndexSearcher searcher = new IndexSearcher(FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryBM25Mesh"))));
		BM25FParameters.setAverageLength(Main.properties.getProperty("docFieldTitle"), Float.parseFloat(Main.properties
				.getProperty("BM25AverageTitleLength")));
		BM25FParameters.setAverageLength(Main.properties.getProperty("docFieldAbstract"), Float
				.parseFloat(Main.properties.getProperty("BM25AverageAbstractLength")));
		BM25FParameters.setK1(2.0f);
		searcher.setSimilarity(new BM25Similarity());
		fields = new String[] {
				Main.properties.getProperty("docFieldTitle", "title"),
				Main.properties.getProperty("docFieldAbstract", "abstract"),
		};
		float[] bParams = new float[fields.length];
		for (int i = 0; i < fields.length; i++) {
			bParams[i] = 0.75f;
		}
		BM25FParameters.setBParam(bParams);
		boosts = new HashMap<String, Float>();
		boosts.put(fields[0], Float.parseFloat(Main.properties.getProperty("titleBoost")));
		boosts.put(fields[1], Float.parseFloat(Main.properties.getProperty("abstractBoost")));
		return searcher;
	}

	@Override
	public Query parseQuery(String query) throws ParseException {
		char[] queryArray = query.toCharArray();
		chunking = chunker.chunk(queryArray, 0, queryArray.length);
		return super.parseQuery(query);
	}

}