package no.ntnu.jervidal.master.medline.searchers;

import java.io.IOException;
import java.util.LinkedList;
import java.util.List;

import no.ntnu.jervidal.master.Main;

import org.apache.log4j.Logger;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.search.Searchable;
import org.apache.lucene.search.TopDocs;

public abstract class MedlineSearcher {
	protected Logger logger = Logger.getLogger(this.getClass());
	protected IndexSearcher indexSearcher;
	int numResults = -1;

	public void initialize() {
		try {
			indexSearcher = initializeIndexSearcher();
		} catch (CorruptIndexException e) {
			logger.error("The index is corrupt.", e);
		} catch (IOException e) {
			logger.error("Could not read the index.", e);
		}
	}

	protected abstract IndexSearcher initializeIndexSearcher() throws CorruptIndexException, IOException;

	public abstract Query parseQuery(String query) throws ParseException;

	public TopDocs search(String query) throws ParseException, IOException {
		if (indexSearcher == null)
			throw new IllegalStateException("Call initialize() before using");
		if (numResults == -1) {
			try {
				numResults = Integer.parseInt(Main.properties.getProperty("numberOfResults"));
			} catch (NumberFormatException e) {
				logger.warn("\"numberOfResults\" was not an Integer. Defaulting to 1000.");
				numResults = 1000;
			}
		}
		return indexSearcher.search(parseQuery(query), numResults);
	}

	public List<TopDocs> search(List<String> queries) throws IOException, ParseException {
		List<TopDocs> results = new LinkedList<TopDocs>();
		for (String query : queries) {
			results.add(search(query));
		}
		return results;
	}

	/**
	 * <strong>Description copied from the interface method:
	 * {@link Searchable#doc(int)}</strong>. Returns the stored fields of
	 * document i.
	 * 
	 * @param docID
	 *            the ID of the document.
	 * @return The {@link Document}.
	 * @throws CorruptIndexException
	 *             if the index is corrupt
	 * @throws IOException
	 *             if there is a low-level IO error
	 */
	public Document getDocument(int docID) throws CorruptIndexException, IOException {
		return indexSearcher.doc(docID);
	}
}
