package no.ntnu.jervidal.master.medline.searchers;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import no.ntnu.jervidal.master.Main;
import no.ntnu.jervidal.master.medline.analyzers.AnalyzerFactory;

import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.queryParser.QueryParser.Operator;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.Query;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.util.Version;

public class MeshPhrase extends MedlineSearcher {

	QueryParser parser;

	@Override
	public IndexSearcher initializeIndexSearcher() throws CorruptIndexException, IOException {
		String dirString = Main.properties.getProperty("indexDirectory");
		return new IndexSearcher(FSDirectory.open(new File(dirString
				+ (dirString.endsWith(File.separator) ? "" : File.separator)
				+ Main.properties.getProperty("indexDirectoryMesh"))));
	}

	@Override
	public Query parseQuery(String query) throws ParseException {
		if (parser == null) {
			String[] fields = new String[] {
					Main.properties.getProperty("docFieldTitle", "title"),
					Main.properties.getProperty("docFieldAbstract", "abstract"),
					Main.properties.getProperty("docFieldMesh", "mesh")
			};
			Map<String, Float> boosts = new HashMap<String, Float>();
			boosts.put(fields[0], Float.parseFloat(Main.properties.getProperty("titleBoost")));
			boosts.put(fields[1], Float.parseFloat(Main.properties.getProperty("abstractBoost")));
			boosts.put(fields[2], Float.parseFloat(Main.properties.getProperty("meshBoost")));
			parser = new MultiFieldQueryParser(Version.LUCENE_30, fields, AnalyzerFactory.getStandardMedlineAnalyzer(),
					boosts);
			parser.setDefaultOperator(Operator.OR);
		}
		return parser.parse(query);
	}
}
